@@ -2,7 +2,7 @@ from urllib.parse import quote
 from os.path import join
 import requests
 
-__version__ = "2.0.0.82"
+__version__ = "2.0.0.89"
 
 _pma_url_content = {}
 _pma_debug = False
@@ -34,4 +34,17 @@ def _pma_http_get(url, headers):
 	
 def _pma_clear_url_cache():
 	global _pma_url_content
-	_pma_url_content = {}
\ No newline at end of file
+	_pma_url_content = {}
+	
+def _pma_set_debug_flag(flag):
+	"""
+	Determine whether Core module runs in debugging mode or not.
+	When in debugging mode (flag = true), extra output is produced when certain conditions in the code are not met
+	"""
+	global _pma_debug
+	
+	if not isinstance(flag, (bool)):
+		raise Exception("flag argument must be of class bool")
+	_pma_debug = flag
+	if flag == True:
+		print("Debug flag enabled. You will receive extra feedback and messages from pma_python (like this one)")
